@@ -2,7 +2,6 @@
 use crate::reference_tables;
 use crate::structs::*;
 use crate::transformation::*;
-use doc_cfg::doc_cfg;
 use std::cmp::Ordering;
 use std::fmt;
 use std::sync::atomic::{AtomicUsize, Ordering as AtomicOrdering};
@@ -102,15 +101,6 @@ impl Atom {
         (self.x, self.y, self.z)
     }
 
-    /// Get the position of the atom as an array of `f64`, in the following order: [x, y, z].
-    /// Given in Aͦ as defined by PDB, to be specific in the orthogonal coordinate system.
-    /// This function is only included for use with `Point` from rstar, so it will be removed
-    /// as soon as rstar implements `Point` for Tuples (https://github.com/georust/rstar/pull/57).
-    #[doc_cfg(feature = "rstar")]
-    pub fn pos_array(&self) -> [f64; 3] {
-        [self.x, self.y, self.z]
-    }
-
     /// Set the position of the atom as a tuple of `f64`, in the following order: (x, y, z).
     /// It fails if one or more of the numbers is not finite (`f64.is_finite()`).
     pub fn set_pos(&mut self, new_pos: (f64, f64, f64)) -> Result<(), String> {
@@ -307,9 +297,9 @@ impl Atom {
     /// It fails when `self.element()` is not an element in the periodic table, or if `self.element()` is undefined and `self.name()` is not an element in the periodic table.
     pub fn atomic_number(&self) -> Option<usize> {
         if !self.element.is_empty() {
-            reference_tables::get_atomic_number(&self.element())
+            reference_tables::get_atomic_number(self.element())
         } else {
-            reference_tables::get_atomic_number(&self.name())
+            reference_tables::get_atomic_number(self.name())
         }
     }
 
@@ -398,7 +388,7 @@ impl Atom {
     }
 
     /// Get if this atom is likely to be a part of the backbone of a protein.
-    /// This is based on this Atom only, for a more precise definition use [AtomWithHierarchy]`.is_backbone()`.
+    /// This is based on this Atom only, for a more precise definition use [hierarchy::ContainsAtomConformer]`.is_backbone()`.
     pub fn is_backbone(&self) -> bool {
         reference_tables::is_backbone(self.name())
     }
@@ -566,18 +556,18 @@ use rstar::{PointDistance, RTreeObject, AABB};
 
 #[cfg(feature = "rstar")]
 impl RTreeObject for &Atom {
-    type Envelope = AABB<[f64; 3]>;
+    type Envelope = AABB<(f64, f64, f64)>;
 
     fn envelope(&self) -> Self::Envelope {
-        AABB::from_point([self.x(), self.y(), self.z()])
+        AABB::from_point(self.pos())
     }
 }
 
 #[cfg(feature = "rstar")]
 impl PointDistance for &Atom {
-    fn distance_2(&self, other: &[f64; 3]) -> f64 {
+    fn distance_2(&self, other: &(f64, f64, f64)) -> f64 {
         // No square root as that is required by the package
-        (other[0] - self.x).powi(2) + (other[1] - self.y).powi(2) + (other[2] - self.z).powi(2)
+        (other.0 - self.x).powi(2) + (other.1 - self.y).powi(2) + (other.2 - self.z).powi(2)
     }
 }
 
@@ -679,7 +669,7 @@ mod tests {
         assert!(a.set_occupancy(0.0).is_ok());
         assert!(a.set_b_factor(0.0).is_ok());
         a.set_hetero(true);
-        assert_eq!(a.hetero(), true);
+        assert!(a.hetero());
         a.set_serial_number(42);
         assert_eq!(a.serial_number(), 42);
         assert_eq!(a.atomic_number(), Some(6));
